#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	//ofBackground(176,255,45);
	ofBackground(0, 191, 136);
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	picture.loadImage("ant.png");
	picture2.loadImage("crumbs.png");
	
	for (int i = 0; i < 100; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
	//	myParticle.setInitialCondition(i,0,0,0);
		particles.push_back(myParticle);
	}
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());
	//VF.randomizeField(2.0);
	
	drawingStyle = 0;
	bFade = true;
}

//--------------------------------------------------------------
void testApp::update(){

	// on every frame
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		
		// if particles are near each other, apply repulsion force
		for (int j = 0; j < i; j++){
			float particlesDistance = ofDist(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			particles[i].addRepulsionForce(particles[j].pos.x, particles[j].pos.y, 50, .1);
		
		} // end for j
		
		// get the force from the vector field: 
		ofxVec2f frc;
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		//cout << frc.x << "  " << frc.y << endl;
		particles[i].force = frc;
		particles[i].addForce(frc.x, frc.y);
		particles[i].addDampingForce();
		particles[i].update();
		
	} // end for i
	
	
	if (bFade == true) VF.fadeField(0.99f);

}

int testApp::nearestCrumb(particle p) {
	float shortestDistance = 99999;
	int crumbPosition = 0;
	for (int i = 0; i < crumbs.size(); i++) {
		if (ofDist(p.pos.x, p.pos.y, crumbs[i].x, crumbs[i].y) < shortestDistance) {
			crumbPosition = i;
			shortestDistance = ofDist(p.pos.x, p.pos.y, crumbs[i].x, crumbs[i].y);
		}
	}
	return crumbPosition;
}
//--------------------------------------------------------------
void testApp::draw(){
	
	ofEnableAlphaBlending();
	ofSetColor(0,130,130, 200);
	//VF.draw();
	ofDisableAlphaBlending();
	
	
	ofSetColor(0xffffff);
	ofEnableAlphaBlending();
	
	// draw crumb images
	for (int c = 0; c < crumbs.size(); c++) {
		picture2.draw(crumbs[c].x, crumbs[c].y);	
	}
	
	
	for (int i = 0; i < particles.size(); i++){
		if (crumbs.size() > 0) {
			
			particles[i].nearestCrumb = nearestCrumb(particles[i]);
			particles[i].addAttractionForce(crumbs[particles[i].nearestCrumb].x, crumbs[particles[i].nearestCrumb].y, 2000, .9f);
		
		}
		
		particles[i].draw(&picture);
	}
	ofDisableAlphaBlending();
	
/*	ofSetColor(0,130,130, 200);
	ofRect(30,30,300,75);
	ofSetColor(255,255,255, 230);
	ofDrawBitmapString("space to clear\nchange drawing mode 'a'\ntoggle fade 'f'", 50, 50);
	
	
	ofSetColor(255,255,130, 230);
	switch (drawingStyle){
		case 0: ofDrawBitmapString("drawing mode: inward", 50, 90);
			break;
		case 1: ofDrawBitmapString("drawing mode: outward", 50, 90);
			break;
		case 2: ofDrawBitmapString("drawing mode: clockwise", 50, 90);
			break;
		case 3: ofDrawBitmapString("drawing mode: counter-clockwise", 50, 90); 
			break;
	}	
*/
	

}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	if (key == ' '){
		VF.clear();
	} else if (key == 'a'){
		drawingStyle ++;
		drawingStyle %= 4;
	} else if (key == 'f'){
		bFade = !bFade;
	} else if (key == 'c'){
		ofPoint tmp;
		tmp.set(ofRandom(20, ofGetWidth()-20), ofRandom(20, ofGetHeight()-20));
		crumbs.push_back(tmp);
		VF.addInwardCircle(float(tmp.x), float(tmp.y), 200, 0.5f);

	}
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (button == 0) {
		switch (drawingStyle){
			case 0: VF.addInwardCircle((float)x, (float)y, 200, 0.3f);
				break;
			case 1: VF.addOutwardCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 2: VF.addClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 3: VF.addCounterClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
		}		
	} else {
		
		particles.erase(particles.begin());
		particle myParticle;
		myParticle.setInitialCondition(x,y,0,0);
		particles.push_back(myParticle);
		
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
